#pragma once

#include <atomic>
#include <thread>

#include <Varjo_plugin.h>

/**
 * Class implementing tracker functionality
 */
class ExampleTracker
{
public:
    /**
     * Initializes tracker
     */
    ExampleTracker(                                      //
        const varjo_PluginUtilityAPI* pluginUtilityAPI,  //
        varjo_PluginUtilitySession pluginUtilitySession);

    /**
     * Starts tracker processing
     */
    varjo_PluginStatus start();

    /**
     * Stops tracker processing
     */
    varjo_PluginStatus stop();

    /**
     * Returns tracked HMD pose
     */
    varjo_PluginStatus getHMDPose(varjo_Nanoseconds timeStamp, varjo_DevicePose* pose);

private:
    /**
     * Main processing loop of the tracker
     */
    void trackerMainLoop();

    const varjo_PluginUtilityAPI* m_pluginUtilityAPI = nullptr;   //!< Pointer to the PluginUtilityAPI
    varjo_PluginUtilitySession m_pluginUtilitySession = nullptr;  //!< PluginUtilityAPI session handle
    int64_t m_frameNumber = 0;                                    //!< Tracker "frame number" - a running counter for tracker results
    varjo_Nanoseconds m_startTime = 0;                            //!< Tracker epoch start
    std::thread m_processingThread;                               //!< Thread running tracker processing
    std::atomic_bool m_isRunning{false};                          //!< Flag indicating tracker processing being active
};
